package java.time.chrono;

import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.format.ResolverStyle;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.time.temporal.ValueRange;
import java.time.temporal.ChronoField;
import java.util.Arrays;
import java.util.List;
import java.time.Instant;
import java.time.temporal.TemporalAccessor;
import java.time.ZoneId;
import java.time.Clock;
import java.time.LocalDate;
import java.util.HashMap;
import java.io.Serializable;

public final class ThaiBuddhistChronology extends AbstractChronology implements Serializable
{
    public static final ThaiBuddhistChronology INSTANCE;
    private static final long serialVersionUID = 2775954514031616474L;
    static final int YEARS_DIFFERENCE = 543;
    private static final HashMap<String, String[]> ERA_NARROW_NAMES;
    private static final HashMap<String, String[]> ERA_SHORT_NAMES;
    private static final HashMap<String, String[]> ERA_FULL_NAMES;
    private static final String FALLBACK_LANGUAGE = "en";
    private static final String TARGET_LANGUAGE = "th";
    
    private ThaiBuddhistChronology() {
    }
    
    @Override
    public String getId() {
        return "ThaiBuddhist";
    }
    
    @Override
    public String getCalendarType() {
        return "buddhist";
    }
    
    @Override
    public ThaiBuddhistDate date(Era era, int n, int n2, int n3) {
        return this.date(this.prolepticYear(era, n), n2, n3);
    }
    
    @Override
    public ThaiBuddhistDate date(int n, int month, int dayOfMonth) {
        return new ThaiBuddhistDate(LocalDate.of(n - 543, month, dayOfMonth));
    }
    
    @Override
    public ThaiBuddhistDate dateYearDay(Era era, int n, int n2) {
        return this.dateYearDay(this.prolepticYear(era, n), n2);
    }
    
    @Override
    public ThaiBuddhistDate dateYearDay(int n, int dayOfYear) {
        return new ThaiBuddhistDate(LocalDate.ofYearDay(n - 543, dayOfYear));
    }
    
    @Override
    public ThaiBuddhistDate dateEpochDay(long epochDay) {
        return new ThaiBuddhistDate(LocalDate.ofEpochDay(epochDay));
    }
    
    @Override
    public ThaiBuddhistDate dateNow() {
        return this.dateNow(Clock.systemDefaultZone());
    }
    
    @Override
    public ThaiBuddhistDate dateNow(ZoneId zone) {
        return this.dateNow(Clock.system(zone));
    }
    
    @Override
    public ThaiBuddhistDate dateNow(Clock clock) {
        return this.date(LocalDate.now(clock));
    }
    
    @Override
    public ThaiBuddhistDate date(TemporalAccessor temporal) {
        if ((temporal instanceof ThaiBuddhistDate)) {
            return (ThaiBuddhistDate)temporal;
        }
        return new ThaiBuddhistDate(LocalDate.from(temporal));
    }
    
    @Override
    public ChronoLocalDateTime<ThaiBuddhistDate> localDateTime(TemporalAccessor temporalAccessor) {
        return super.localDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<ThaiBuddhistDate> zonedDateTime(TemporalAccessor temporalAccessor) {
        return super.zonedDateTime(temporalAccessor);
    }
    
    @Override
    public ChronoZonedDateTime<ThaiBuddhistDate> zonedDateTime(Instant instant, ZoneId zoneId) {
        return super.zonedDateTime(instant, zoneId);
    }
    
    @Override
    public boolean isLeapYear(long n) {
        return IsoChronology.INSTANCE.isLeapYear(n - 543L);
    }
    
    @Override
    public int prolepticYear(Era era, int n) {
        if (!(era instanceof ThaiBuddhistEra)) {
            throw new ClassCastException("Era must be BuddhistEra");
        }
        return (era == ThaiBuddhistEra.BE) ? n : (1 - n);
    }
    
    @Override
    public ThaiBuddhistEra eraOf(int n) {
        return ThaiBuddhistEra.of(n);
    }
    
    @Override
    public List<Era> eras() {
        return Arrays.asList(ThaiBuddhistEra.values());
    }
    
    @Override
    public ValueRange range(ChronoField chronoField) {
        switch (chronoField) {
            case PROLEPTIC_MONTH: {
                ValueRange range = ChronoField.PROLEPTIC_MONTH.range();
                return ValueRange.of(range.getMinimum() + 6516L, range.getMaximum() + 6516L);
            }
            case YEAR_OF_ERA: {
                ValueRange range2 = ChronoField.YEAR.range();
                return ValueRange.of(1L, -(range2.getMinimum() + 543L) + 1L, range2.getMaximum() + 543L);
            }
            case YEAR: {
                ValueRange range3 = ChronoField.YEAR.range();
                return ValueRange.of(range3.getMinimum() + 543L, range3.getMaximum() + 543L);
            }
            default: {
                return chronoField.range();
            }
        }
    }
    
    @Override
    public ThaiBuddhistDate resolveDate(Map<TemporalField, Long> map, ResolverStyle resolverStyle) {
        return (ThaiBuddhistDate)super.resolveDate(map, resolverStyle);
    }
    
    @Override
    Object writeReplace() {
        return super.writeReplace();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    static {
        INSTANCE = new ThaiBuddhistChronology();
        ERA_NARROW_NAMES = new HashMap();
        ERA_SHORT_NAMES = new HashMap();
        ERA_FULL_NAMES = new HashMap();
        ThaiBuddhistChronology.ERA_NARROW_NAMES.put("en", new String[] { "BB", "BE" });
        ThaiBuddhistChronology.ERA_NARROW_NAMES.put("th", new String[] { "BB", "BE" });
        ThaiBuddhistChronology.ERA_SHORT_NAMES.put("en", new String[] { "B.B.", "B.E." });
        ThaiBuddhistChronology.ERA_SHORT_NAMES.put("th", new String[] { "พ.ศ.", "ปีก่อนคริสต์กาลที่" });
        ThaiBuddhistChronology.ERA_FULL_NAMES.put("en", new String[] { "Before Buddhist", "Budhhist Era" });
        ThaiBuddhistChronology.ERA_FULL_NAMES.put("th", new String[] { "พุทธศักราช", "ปีก่อนคริสต์กาลที่" });
    }
}
